import 'dart:async';
import 'package:flutter/material.dart';
import 'package:timeago_text/scr/lang/en.dart';
import 'package:timeago_text/scr/lang/en_short.dart';
import 'package:timeago_text/scr/lang/th.dart';
import 'package:timeago_text/scr/lang/th_short.dart';

class TimeAgoText extends StatefulWidget {
  final DateTime date;
  final String lang;
  final Locale locale;
  final int maxLines;
  final TextOverflow overflow;
  final String semanticsLabel;
  final bool softWarp;
  final StrutStyle strutStyle;
  final TextStyle style;
  final TextAlign textAlign;
  final TextDirection textDirection;
  final TextHeightBehavior textHeightBehavior;
  final double textScaleFactor;
  final TextWidthBasis textWidthBasis;
  TimeAgoText(
      {this.date,
      this.lang,
      this.locale,
      this.maxLines,
      this.overflow,
      this.semanticsLabel,
      this.softWarp,
      this.strutStyle,
      this.style,
      this.textAlign,
      this.textDirection,
      this.textHeightBehavior,
      this.textScaleFactor,
      this.textWidthBasis})
      : super(key: UniqueKey());
  @override
  TimeAgoTextState createState() => TimeAgoTextState();
}

class TimeAgoTextState extends State<TimeAgoText> {
  String text = "";
  @override
  void initState() {
    super.initState();
    _autoUpdateTimeAgo();
  }

  @override
  Widget build(BuildContext context) {
    return Text(
      text,
      locale: widget.locale,
      maxLines: widget.maxLines,
      overflow: widget.overflow,
      semanticsLabel: widget.semanticsLabel,
      softWrap: widget.softWarp,
      strutStyle: widget.strutStyle,
      style: widget.style,
      textAlign: widget.textAlign,
      textDirection: widget.textDirection,
      textHeightBehavior: widget.textHeightBehavior,
      textScaleFactor: widget.textScaleFactor,
      textWidthBasis: widget.textWidthBasis,
    );
  }

  void _autoUpdateTimeAgo() {
    Map langText;
    int timeDiff = widget.date.difference(DateTime.now()).inSeconds.abs();
    if (widget.lang == null || widget.lang == "en") {
      langText = en();
    } else if (widget.lang == "en_short") {
      langText = enShort();
    } else if (widget.lang == "th") {
      langText = th();
    } else if (widget.lang == "th_short") {
      langText = thShort();
    }
    if (widget.date != null) {
      if (timeDiff < 60) {
        text = langText["now"];
        int timeToRefresh = 61 - timeDiff;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff < 3600) {
        text = (timeDiff / 60).floor().toString() + " " + langText["min"];
        if (timeDiff < 120) {
          text = langText["amin"];
        }
        int timeToRefresh = 61 - timeDiff % 60;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff < 86400) {
        text = (timeDiff / 3600).floor().toString() + " " + langText["hr"];
        if (timeDiff < 7200) {
          text = langText["ahr"];
        }
        int timeToRefresh = 3601 - timeDiff % 3600;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff < 604800) {
        text = (timeDiff / 86400).floor().toString() + " " + langText["d"];
        if (timeDiff < 172800) {
          text = langText["ad"];
        }
        int timeToRefresh = 86401 - timeDiff % 86400;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff < 2592000) {
        text = (timeDiff / 604800).floor().toString() + " " + langText["wk"];
        if (timeDiff < 1209600) {
          text = langText["awk"];
        }
        int timeToRefresh = 604801 - timeDiff % 604800;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff < 31536000) {
        text = (timeDiff / 2592000).floor().toString() + " " + langText["mth"];
        if (timeDiff < 518400) {
          text = langText["amth"];
        }
        int timeToRefresh = 2592001 - timeDiff % 2592000;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      } else if (timeDiff > 31536000) {
        text = (timeDiff / 31536000).floor().toString() + " " + langText["yr"];
        if (timeDiff < 630720000) {
          text = langText["ayr"];
        }
        int timeToRefresh = 31536001 - timeDiff % 31536000;
        Timer.periodic(Duration(seconds: timeToRefresh), (Timer t) {
          if (this.mounted) {
            setState(() {
              _autoUpdateTimeAgo();
            });
          }
        });
      }
    }
  }
}
